/**
 * misc.c
 * Belongs to the TARGETING software
 * http://www.bioinf.uni-leipzig.de/Software/Targeting
 *
 * @author Christian Arnold
 * @email achristian@bioinf.uni-leipzig.de
 * @company Professur fuer Bioinformatik, University of Leipzig
 */
 
   
/*           TARGETING Copyright (C) 2010  Christian Arnold

* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>
* or the file "GNU Public License.txt" in the "src" folder.
*/

#include "misc.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>


int **allocateMemoryMatrixInt(int nrows, int ncols) {
	int **array;
	
	array = malloc(nrows * sizeof(int *));
	if(array == NULL) {
		fprintf(stderr, "Error: Could not allocate memory, exit...\n");
		exit(EXIT_FAILURE);
	}
	for(int i = 0; i < nrows; i++) {
		array[i] = malloc(ncols * sizeof(int));
		if(array[i] == NULL){
			fprintf(stderr, "Error: Could not allocate memory, exit...\n");
			exit(EXIT_FAILURE);
		}
	}
	return array;
}

double **allocateMemoryMatrixFloat(int nrows, int ncols) {
	double **array;
	
	array = malloc(nrows * sizeof(double *));
	if(array == NULL) {
		fprintf(stderr, "Error: Could not allocate memory, exit...\n");
		exit(EXIT_FAILURE);
	}
	for(int i = 0; i < nrows; i++) {
		array[i] = malloc(ncols * sizeof(double));
		if(array[i] == NULL){
			fprintf(stderr, "Error: Could not allocate memory, exit...\n");
			exit(EXIT_FAILURE);
		}
	}
	return array;
}

Node ***allocateMemoryMatrixNode(int nrows, int ncols) {
	Node ***array;
	
	array = malloc(nrows * sizeof(Node**));
	if(array == NULL) {
		fprintf(stderr, "Error: Could not allocate memory, exit...\n");
		exit(EXIT_FAILURE);
	}
	for(int i = 0; i < nrows; i++) {
		array[i] = malloc(ncols * sizeof(Node*));
		if(!array[i]){
			fprintf(stderr, "Error: Could not allocate memory, exit...\n");
			exit(EXIT_FAILURE);
		}
	}
	return array;
}

void initMatrixInt(int **array, int nrows, int ncols, int value) {
	for(int i = 0; i < nrows; i++) {
		for(int j = 0; j < ncols; j++)
			array[i][j] = value;
	}
}

void initMatrixFloat(double **array, int nrows, int ncols, int value) {
	for(int i = 0; i < nrows; i++) {
		for(int j = 0; j < ncols; j++)
			array[i][j] = value;
	}
}

void initMatrixNode(Node ***array, int nrows, int ncols, Node *value) {
	for(int i = 0; i < nrows; i++) {
		for(int j = 0; j < ncols; j++)
			array[i][j] = value;
	}
}

void initArrayFloat (double *array, int nrows, int value) {
     for(int i = 0; i < nrows; i++) {
	array[i] = value;
     }
}

void printMatrixInt(int **array, int nrows, int ncols) {
	printf("Print matrix...\n");
	for (int i = 0; i < nrows; i++) {
		printf("Row %d: ",i);
		for (int j = 0; j < ncols; j++) {
		      if (array[i][j] >= 0) {
					printf(" ");
		      }
		      
		      printf("%d ",array[i][j]);

		}
		printf("\n");
	}
}

void printMatrixFloat(double **array, int nrows, int ncols) {
	printf("Print matrix...\n");
	for (int i = 0; i < nrows; i++) {
		printf("Row %d: ",i);
		for (int j = 0; j < ncols; j++) {
		      if (array[i][j] >= 0) {
				printf(" ");
		      }
		      printf("%f ",array[i][j]);

		}
		printf("\n");
	}
}

void printArrayFloat (double *array, int nrows) {
     printf("Print array...\n");
     for (int i = 0; i < nrows; i++) {
	  printf("Entry %d: %f\n",i,array[i]);
     }
}

void printMatrixNode(Node ***array, int nrows, int ncols) {
	printf("Print matrix N...\n");
	for (int i = 0; i < nrows; i++) {
		printf("Row %d: ",i);
		for (int j = 0; j < ncols; j++) {
		      if (array[i][j] == NULL) {
			  printf(" N "); 
		      } else {
			 if (array[i][j]->idA >= 0) {
				   printf(" ");
			 }
			 printf("%d ",array[i][j]->idA);
		      }

		}
		printf("\n");
	}	     
	     
}

void printArrayF (Node ***array, int nrows, int ncols) {
     printf("Print F array...\n");
     for (int i = 0; i < nrows; i++) {
	  printf("F[idI = %d]: ",i);
	  for (int j = 0; j < ncols; j++) {
	       if (array[i][j] == NULL) {
		    printf("No pair"); 
		    break;
	       } else {
		    printf("%s ",array[i][j]->p_label);
	       }

	  }
	  printf("\n");
     }
}

void freeMemoryMatrix (void **array, int nrows) {
	for(int i = 0; i < nrows; i++)
		free(array[i]);
	free(array);
}

void freeMemoryMatrix2 (Node ***array, int nrows) {
	for(int i = 0; i < nrows; i++)
		free(array[i]);
	free(array);
}


/*
Imitates substr function from PHP and Perl
*/
char *substring(const char *str, int begin, int len) {
  
	//printf("  Calling substring procedure with string %s, begin at pos %d, length=%d\n",str,begin,len);
	char *dst;
	dst = malloc((len+1) * sizeof(char));
	strncpy(dst,str+begin,len);
	dst[len]='\0';
	if (strlen(dst)!=len) {
	  fprintf(stderr," Error in substring procedure: Length not %d, but %d\n",len,strlen(dst));
	  exit(EXIT_FAILURE);
	}
  	return dst;
}

char *trimWhitespace(char *str) {
  char *end;

  // Trim leading space
  while(isspace(*str)) str++;

  // Trim trailing space
  end = str + strlen(str) - 1;
  while(end > str && isspace(*end)) end--;

  // Write new null terminator
  *(end+1) = 0;

  return str;

}


int isValidSpName (char *sp, char **arr_allSp, int numberSpecies) {

	for (int i=0;i<numberSpecies;i++) {
		if (strcmp(arr_allSp[i],sp) == 0) {
			return 1;
		}
	}
	
	return 0;
}

