/* File tree.h.  */
#ifndef FILE_TREE_SEEN
#define FILE_TREE_SEEN

	//use typedef to avoid the prefix struct
	typedef struct node{
		int idA; /* ID if all nodes are considered */
		int idL; /* ID if only leaves are considered */
		int idI; /* ID if only interior vertices are considered */
		int idM; /* ID if only multifurcating vertices are considered */
		int deg;
		struct node *p_nextLeaf; //NULL if not a tip
		char *p_label;
		struct node *p_parent;
		struct node **p_children; //Pointer auf dynamisches Array, dass alle children enthaelt
	} Node;

	void freeTree (Node *p_node);

	void printNode (Node *node, int spaces);

	Node** getNextLeafRek(Node *p_node, Node **pp_leafList,int *counterLeaves,int *counterInternal);
	
	void getTaxaNames (char **taxaNames, Node** pp_leafList, int numberTaxa);
	
	Node* getUquer (Node *p_internal, Node *p_leaf, Node ***N);

	void printTree (Node *root);

	void printTreeRek (Node *root, int spaces);
	
	int isBinary(Node *p_node);
	
	void traverseTreePostorder (Node **p_nodes, Node *p_leaf,int *p_nodeCounter);
	
	Node ** getLeavesInSubtree (Node *p_node, Node **pp_leafList, int *p_counterInternal);
	
	void initIdM(Node *p_node, int *counterIdM);
	
	void mallocMatricesMultiFurcating (Node *p_node, double ***Q2, Node *****F2,  int ***MWM, int *counter_idM);
	
	void initMatricesMultiFurcating (Node *p_node, double ***Q2, Node *****F2, int ***MWM, int *counter_idM);
	
	void freeMatricesMultiFurcating (Node *p_node, Node *****F2, double ***Q2, int ***MWM, int *counter_idM);
	
	void initScoresMode (Node *p_root, double **W, Node **pp_leafList, int numberLeaves);
	
	int getTreeDepth (Node *p_root, Node **pp_leafList, int numberLeaves);
	

#endif /* !FILE_TREE_SEEN */